#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Dict

from fbpcp.entity.cloud_provider import CloudProvider
from onedocker.repository.onedocker_repository_service import OneDockerRepositoryService

from onedocker.util.service_builder import build_repository_service


class RepositoryServiceGateway:
    def __init__(self) -> None:
        self.repository_service: OneDockerRepositoryService = build_repository_service(
            cloud_provider=CloudProvider.AWS
        )

    def get_measurements(self, package_name: str, version: str) -> Dict[str, str]:
        return self.repository_service.get_package_measurements(package_name, version)



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass, field
from typing import Dict

from dataclasses_json import config, DataClassJsonMixin

from onedocker.entity.opawdl_state import OPAWDLState


@dataclass
class OPAWDLWorkflow(DataClassJsonMixin):
    starts_at: str = field(metadata=config(field_name="StartAt"))
    states: Dict[str, OPAWDLState] = field(metadata=config(field_name="States"))

    def __str__(self) -> str:
        return self.to_json()



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from functools import reduce
from typing import Any, Dict, List, Optional

import boto3
from botocore.client import BaseClient
from fbpcp.decorator.error_handler import error_handler
from fbpcp.entity.secret import StringSecret
from fbpcp.gateway.aws import AWSGateway
from fbpcp.util.aws import convert_list_to_dict


class AWSSecretsManagerGateway(AWSGateway):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config)
        self.client: BaseClient = boto3.client(
            "secretsmanager", region_name=self.region, **self.config
        )

    @error_handler
    def create_secret(
        self,
        secret_name: str,
        secret_value: str,
        tags: Optional[Dict[str, str]] = None,
    ) -> str:
        """
        Returns the id (ARN) of the created secret
        """
        tags_dict = []
        if tags:
            tags_dict = self._generate_tags_dict(tags)

        response = self.client.create_secret(
            Name=secret_name, SecretString=secret_value, Tags=tags_dict
        )
        return response["ARN"]

    @error_handler
    def get_secret(
        self,
        secret_id: str,
    ) -> StringSecret:
        # Get secret value.
        # It retrieves the current version of the secret.
        val_response = self.client.get_secret_value(SecretId=secret_id)
        # Get secret details. E.g tags
        descr_response = self.client.describe_secret(SecretId=secret_id)

        return self._convert_resp_to_secret(descr_response, val_response)

    @error_handler
    def delete_secret(
        self,
        secret_id: str,
    ) -> None:
        # Delete secret.
        self.client.delete_secret(SecretId=secret_id)

    def _convert_resp_to_secret(
        self, descr_resp: Dict[str, Any], val_resp: Dict[str, Any]
    ) -> StringSecret:
        """
        Encapsulate the responses into a Secret object
        """
        id = descr_resp["ARN"]
        name = descr_resp["Name"]
        value = val_resp.get("SecretString")
        create_date = descr_resp.get("CreatedDate")
        tags = convert_list_to_dict(descr_resp.get("Tags"), "Key", "Value")

        return StringSecret(
            id=id, name=name, value=value, create_date=create_date, tags=tags
        )

    def _generate_tags_dict(self, tags: Dict[str, str]) -> List[Dict[str, str]]:
        # Input tag format {"tag1": "v1", "tag2", "v2", ...}
        # AWS required format [{"Key": "tag1", "Value": "v1"}, ...}
        new_dict = reduce(
            lambda x, y: [*x, {"Key": y, "Value": tags[y]}], tags.keys(), []
        )

        return new_dict



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass
from enum import Enum
from typing import Dict, Optional

from dataclasses_json import DataClassJsonMixin


class PolicyName(str, Enum):
    BINARY_MATCH = "BINARY_MATCH"


@dataclass
class PolicyParams(DataClassJsonMixin):
    package_name: Optional[str]
    version: Optional[str]


@dataclass
class AttestationPolicy(DataClassJsonMixin):
    policy_name: PolicyName
    params: PolicyParams


@dataclass
class AttestationDocument(DataClassJsonMixin):
    policy: AttestationPolicy
    measurements: Dict[str, str]



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict


from dataclasses import dataclass
from enum import Enum

from dataclasses_json import DataClassJsonMixin
from onedocker.entity.opawdl_state import OPAWDLState


class Status(Enum):
    CREATED = "CREATED"
    STARTED = "STARTED"
    FAILED = "FAILED"
    COMPLETED = "COMPLETED"


@dataclass
class OPAWDLStateInstance(DataClassJsonMixin):
    opawdl_state: OPAWDLState
    status: Status = Status.CREATED

    def __str__(self) -> str:
        return self.to_json()



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import asyncio

from typing import Any, Dict, Optional

from fbpcp.entity.secret import StringSecret

from fbpcp.gateway.secrets_manager import AWSSecretsManagerGateway
from fbpcp.service.secrets_manager import SecretsManagerService


class AWSSecretsManagerService(SecretsManagerService):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        self.secret_gateway = AWSSecretsManagerGateway(
            region, access_key_id, access_key_data, config
        )

    def create_secret(
        self,
        secret_name: str,
        secret_value: str,
        tags: Optional[Dict[str, str]] = None,
    ) -> str:
        secret_id = self.secret_gateway.create_secret(
            secret_name=secret_name, secret_value=secret_value, tags=tags
        )

        return secret_id

    def get_secret(
        self,
        secret_id: str,
    ) -> StringSecret:
        # secret id can be ARN or secret name
        secret = self.secret_gateway.get_secret(secret_id=secret_id)

        return secret

    async def create_secret_async(
        self,
        secret_name: str,
        secret_value: str,
        tags: Optional[Dict[str, str]] = None,
    ) -> str:
        loop = asyncio.get_running_loop()
        result = await loop.run_in_executor(
            None, self.create_secret, secret_name, secret_value, tags
        )
        return result

    async def get_secret_async(self, secret_id: str) -> StringSecret:
        loop = asyncio.get_running_loop()
        result = await loop.run_in_executor(None, self.get_secret, secret_id)
        return result

    def delete_secret(self, secret_id: str) -> None:
        self.secret_gateway.delete_secret(secret_id=secret_id)

    async def delete_secret_async(self, secret_id: str) -> None:
        loop = asyncio.get_running_loop()
        await loop.run_in_executor(None, self.delete_secret, secret_id)



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from onedocker.entity.attestation_document import (
    AttestationDocument,
    AttestationPolicy,
    PolicyName,
)
from onedocker.service.attestation import AttestationService
from onedocker.service.attestation_pc import PCAttestationService


class AttestationFactoryService:
    def validate(self, attestation_document: str) -> bool:
        try:
            document = AttestationDocument.from_json(attestation_document)
        except Exception as e:
            raise ValueError(
                f"Failed to parse attestation document json, reason: {e}"
            ) from e
        attestation_svc = self._get_attestation_service(document.policy)
        return attestation_svc.validate(document.policy, document.measurements)

    def _get_attestation_service(self, policy: AttestationPolicy) -> AttestationService:
        if policy.policy_name == PolicyName.BINARY_MATCH:
            return PCAttestationService()
        else:
            raise NotImplementedError(
                f"Policy name {policy.policy_name} is NOT supported for now."
            )



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc
from typing import Dict, Optional

from fbpcp.entity.secret import StringSecret


class SecretsManagerService(abc.ABC):
    @abc.abstractmethod
    def create_secret(
        self, secret_name: str, secret_value: str, tags: Optional[Dict[str, str]] = None
    ) -> str:
        pass

    @abc.abstractmethod
    def get_secret(
        self,
        secret_id: str,
    ) -> StringSecret:
        pass

    @abc.abstractmethod
    async def create_secret_async(
        self, secret_name: str, secret_value: str, tags: Optional[Dict[str, str]] = None
    ) -> str:
        pass

    @abc.abstractmethod
    async def get_secret_async(
        self,
        secret_id: str,
    ) -> StringSecret:
        pass

    @abc.abstractmethod
    def delete_secret(
        self,
        secret_id: str,
    ) -> None:
        pass

    @abc.abstractmethod
    async def delete_secret_async(
        self,
        secret_id: str,
    ) -> None:
        pass



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict


import logging
from typing import Dict

from fbpcp.error.pcp import InvalidParameterError

from onedocker.entity.attestation_document import AttestationPolicy, PolicyName
from onedocker.gateway.repository_service import RepositoryServiceGateway
from onedocker.service.attestation import AttestationService


class PCAttestationService(AttestationService):
    def __init__(self) -> None:
        self.repository_service_gateway = RepositoryServiceGateway()
        self.logger: logging.Logger = logging.getLogger(__name__)

    def validate(self, policy: AttestationPolicy, measurements: Dict[str, str]) -> bool:
        if policy.policy_name != PolicyName.BINARY_MATCH:
            raise NotImplementedError("Only BINARY_MATCH policy is supported for now.")
        if policy.params.package_name is None or policy.params.version is None:
            raise InvalidParameterError(
                "Package name and version must be specified in policy to perform binary match."
            )
        return self.binary_match(
            policy.params.package_name, policy.params.version, measurements
        )

    def binary_match(
        self, package_name: str, version: str, measurements: Dict[str, str]
    ) -> bool:
        allowlist = self.repository_service_gateway.get_measurements(
            package_name, version
        )
        for measurement_key, measurement_value in measurements.items():
            if measurement_key not in allowlist:
                self.logger.error(
                    f"Cannot find measurement with key '{measurement_key}' in allowlist."
                )
                return False
            if allowlist[measurement_key] != measurement_value:
                self.logger.error(
                    f"Measurement with key '{measurement_key}' does not match the allowed value."
                )
                return False
        return True



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass
from enum import Enum
from typing import List

from dataclasses_json import DataClassJsonMixin
from onedocker.entity.opawdl_state_instance import OPAWDLStateInstance
from onedocker.entity.opawdl_workflow import OPAWDLWorkflow


class Status(Enum):
    CREATED = "CREATED"
    STARTED = "STARTED"
    FAILED = "FAILED"
    COMPLETED = "COMPLETED"


@dataclass
class OPAWDLWorkflowInstance(DataClassJsonMixin):
    instance_id: str
    opawdl_workflow: OPAWDLWorkflow
    state_instances: List[OPAWDLStateInstance]
    status: Status = Status.CREATED

    def get_instance_id(self) -> str:
        return self.instance_id

    def __str__(self) -> str:
        return self.to_json()



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass, field
from typing import List, Optional

from dataclasses_json import config, DataClassJsonMixin


@dataclass
class OPAWDLState(DataClassJsonMixin):
    plugin_name: str = field(metadata=config(field_name="PluginName"))
    cmd_args_list: List[str] = field(metadata=config(field_name="CmdArgsList"))
    timeout: Optional[int] = field(metadata=config(field_name="Timeout"), default=None)
    next_: Optional[str] = field(metadata=config(field_name="Next"), default=None)
    is_end: Optional[bool] = field(metadata=config(field_name="IsEnd"), default=True)

    def __str__(self) -> str:
        return self.to_json()



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import logging
import subprocess
from typing import List

from onedocker.entity.opawdl_state import OPAWDLState
from onedocker.entity.opawdl_state_instance import (
    OPAWDLStateInstance,
    Status as StateStatus,
)

from onedocker.entity.opawdl_workflow import OPAWDLWorkflow
from onedocker.entity.opawdl_workflow_instance import (
    OPAWDLWorkflowInstance,
    Status as WorkflowStatus,
)
from onedocker.repository.opawdl_workflow_instance_repository import (
    OPAWDLWorkflowInstanceRepository,
)
from onedocker.util.opawdl_parser import OPAWDLParser


class OPAWDLDriver:
    """OPAWDLDriver Executes OPAWDL Workflow"""

    def __init__(
        self,
        instance_id: str,
        workflow_path: str,
        repo: OPAWDLWorkflowInstanceRepository,
    ) -> None:
        self.logger: logging.Logger = logging.getLogger(__name__)
        self.workflow_instance_id = instance_id
        self.workflow_instance_repo = repo
        with open(workflow_path, "r") as f:
            parser = OPAWDLParser()
            self.workflow: OPAWDLWorkflow = parser.parse_json_str_to_workflow(f.read())
        # initialize workflow instance
        self.workflow_instance: OPAWDLWorkflowInstance = OPAWDLWorkflowInstance(
            instance_id=self.workflow_instance_id,
            opawdl_workflow=self.workflow,
            state_instances=[],
            status=WorkflowStatus.CREATED,
        )
        self.workflow_instance_repo.create(self.workflow_instance)

    def run_workflow(self) -> None:
        self.logger.info(f"Running workflow: {self.workflow_instance_id}")
        curr_state_name = self.workflow.starts_at
        curr_state = self.workflow.states[self.workflow.starts_at]

        while curr_state is not None:
            # run current state
            self.logger.info(f"Running state: {curr_state_name}")
            self._run_state(curr_state)
            # find next state
            if curr_state.next_ is not None and curr_state.is_end is False:
                curr_state_name = curr_state.next_
                curr_state = self.workflow.states[curr_state.next_]
            else:
                curr_state = None

        self.workflow_instance.status = WorkflowStatus.COMPLETED
        self.workflow_instance_repo.update(self.workflow_instance)
        self.logger.info(f"Workflow {self.workflow_instance_id} completed running")

    def _run_state(self, curr_state: OPAWDLState) -> None:
        curr_state_instance = OPAWDLStateInstance(curr_state, StateStatus.STARTED)

        # execute cmd
        shell_cmd = self._build_shell_cmd(
            plugin_name=curr_state.plugin_name, cmd_args_list=curr_state.cmd_args_list
        )
        subprocess.run(shell_cmd, shell=True)

        # update instances
        curr_state_instance = OPAWDLStateInstance(curr_state, StateStatus.COMPLETED)
        self._add_state_instance_to_workflow_instance(curr_state_instance)

    def _build_shell_cmd(self, plugin_name: str, cmd_args_list: List[str]) -> str:
        return plugin_name + " " + " ".join(cmd_args_list)

    def _add_state_instance_to_workflow_instance(
        self, state_instance: OPAWDLStateInstance
    ) -> None:
        self.workflow_instance.state_instances.append(state_instance)
        self.workflow_instance_repo.update(self.workflow_instance)



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Dict

from fbpcp.entity.cloud_provider import CloudProvider
from fbpcp.service.storage_s3 import S3StorageService
from fbpcp.util.s3path import S3Path
from onedocker.repository.onedocker_repository_service import OneDockerRepositoryService
from onedocker.service.metadata import MetadataService

PROD = "PROD"
STAGING = "STAGING"
REPOSITORY_PATHS: Dict[str, str] = {
    PROD: "https://one-docker-repository-prod.s3.us-west-2.amazonaws.com/",
    STAGING: "https://onedocker-repository-test.s3.us-west-2.amazonaws.com/",
}

METADATA_TABLE_KEY_NAME = "primary_key"
METADATA_TABLES: Dict[str, str] = {STAGING: "metadata_test"}


def build_repository_service(
    cloud_provider: CloudProvider, env: str = "STAGING", unsigned_enabled: bool = False
) -> OneDockerRepositoryService:
    if cloud_provider != CloudProvider.AWS:
        raise NotImplementedError(
            "Only AWS is supported for building Repository Service for now."
        )
    repository_path = REPOSITORY_PATHS[env]
    region = S3Path(repository_path).region
    storage_svc = S3StorageService(
        region,
        unsigned_enabled=unsigned_enabled,
    )
    metadata_svc = MetadataService(
        region=region, table_name=METADATA_TABLES[env], key_name=METADATA_TABLE_KEY_NAME
    )
    return OneDockerRepositoryService(
        storage_svc=storage_svc,
        package_repository_path=repository_path,
        metadata_svc=metadata_svc,
    )



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from pathlib import Path

from onedocker.entity.opawdl_workflow_instance import OPAWDLWorkflowInstance
from onedocker.repository.opawdl_workflow_instance_repository import (
    OPAWDLWorkflowInstanceRepository,
)


class LocalOPAWDLWorkflowInstanceRepository(OPAWDLWorkflowInstanceRepository):
    def __init__(self, base_dir: str) -> None:
        self.base_dir = Path(base_dir)

    def _exist(self, instance_id: str) -> bool:
        return self.base_dir.joinpath(instance_id).exists()

    def create(self, instance: OPAWDLWorkflowInstance) -> None:
        if self._exist(instance.get_instance_id()):
            raise Exception(
                f"Fail to create the workflow instance: {instance.get_instance_id()} already exists."
            )

        path = self.base_dir.joinpath(instance.get_instance_id())
        with open(path, "w") as f:
            f.write(str(instance))

    def get(self, instance_id: str) -> OPAWDLWorkflowInstance:
        if not self._exist(instance_id):
            raise Exception(f"{instance_id} does NOT exist")

        path = self.base_dir.joinpath(instance_id)
        with open(path, "r") as f:
            return OPAWDLWorkflowInstance.from_json((f.read().strip()))

    def update(self, instance: OPAWDLWorkflowInstance) -> None:
        if not self._exist(instance.get_instance_id()):
            raise Exception(f"{instance.get_instance_id()} does not exist")

        path = self.base_dir.joinpath(instance.get_instance_id())
        with open(path, "w") as f:
            f.write(str(instance))

    def delete(self, instance_id: str) -> None:
        if not self._exist(instance_id):
            raise Exception(f"{instance_id} does not exist")

        self.base_dir.joinpath(instance_id).unlink()



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from onedocker.entity.opawdl_workflow import OPAWDLWorkflow


class OPAWDLParser:
    def parse_json_str_to_workflow(
        self,
        input_str: str,
    ) -> OPAWDLWorkflow:
        workflow = OPAWDLWorkflow.from_json(input_str)

        # Validate end state
        has_end = 0
        for state in workflow.states.values():
            if state.is_end:
                has_end += 1
        if has_end == 0:
            raise Exception("Input workflow string does not have an ending state.")
        elif has_end > 1:
            raise Exception(
                f"Input workflow string has multiple({has_end}) ending states."
            )

        return workflow



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from enum import IntEnum


class ExitCode(IntEnum):
    """Custom exit codes for OneDocker containers.

    Unix uses error codes above 125 specially for failures, so they should be avoided for program errors.

    The meaning of the codes is approximately as follows:
        SUCCESS -- The program succeeded.
        EXE_ERROR -- The executable itself exited with non-zero code.
        ERROR -- Catchall for general errors.
        SERVICE_UNAVAILABLE -- A service is unavailable. This can occur if a support program or file is unavailable.
        TIMEOUT -- The program timed out.
        SIGINT -- The program received SIGINT signal.

    For other exit codes please refer to https://man.freebsd.org/cgi/man.cgi?sysexits.
    """

    SUCCESS = 0
    EXE_ERROR = 1
    ERROR = 64
    SERVICE_UNAVAILABLE = 69
    TIMEOUT = 124
    SIGINT = 130



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc
from typing import Dict

from onedocker.entity.attestation_document import AttestationPolicy


class AttestationService(abc.ABC):
    @abc.abstractmethod
    def validate(self, policy: AttestationPolicy, measurements: Dict[str, str]) -> bool:
        pass
